@@ -36,13 +36,13 @@
 #include <QDebug>
 #include <QRandomGenerator>
 
+#include <KConfigGroup>
 #include <KFileItem>
 #include <KLocalizedString>
-#include <KSharedConfig>
-#include <KConfigGroup>
-#include <KMimeTypeTrader>
-#include <KServiceTypeTrader>
+#include <KPluginInfo>
 #include <KPluginLoader>
+#include <KServiceTypeTrader>
+#include <KSharedConfig>
 
 #include <kio/previewjob.h>
 #include <kio/thumbcreator.h>
@@ -109,7 +109,7 @@ extern "C" Q_DECL_EXPORT int kdemain( int argc, char **argv )
     // so it for example wouldn't reply to ksmserver - on the other
     // hand, this slave uses QPixmaps for some reason, and they
     // need QGuiApplication
-    putenv(strdup("SESSION_MANAGER="));
+    qunsetenv("SESSION_MANAGER");
 
     // Some thumbnail plugins use QWidget classes for the rendering,
     // so use QApplication here, not just QGuiApplication
@@ -139,7 +139,6 @@ ThumbnailProtocol::ThumbnailProtocol(const QByteArray &pool, const QByteArray &a
 ThumbnailProtocol::~ThumbnailProtocol()
 {
     qDeleteAll(m_creators);
-    m_creators.clear();
 }
 
 /**
@@ -252,7 +251,7 @@ void ThumbnailProtocol::get(const QUrl &url)
         }
 
         if (creator->handleSequences) {
-            ThumbSequenceCreator* sequenceCreator = dynamic_cast<ThumbSequenceCreator*>(creator->creator);
+            ThumbSequenceCreator* sequenceCreator = dynamic_cast<ThumbSequenceCreator*>(creator->creator.get());
             if (sequenceCreator) {
                 sequenceCreator->setSequenceIndex(sequenceIndex());
 
@@ -266,7 +265,7 @@ void ThumbnailProtocol::get(const QUrl &url)
         }
 
         if (creator->handleSequences) {
-            ThumbSequenceCreator* sequenceCreator = dynamic_cast<ThumbSequenceCreator*>(creator->creator);
+            ThumbSequenceCreator* sequenceCreator = dynamic_cast<ThumbSequenceCreator*>(creator->creator.get());
             // We MUST do this after calling create(), because the create() call itself might change it.
             const float wp = sequenceCreator->sequenceIndexWraparoundPoint();
             setMetaData("sequenceIndexWraparoundPoint", QString().setNum(wp));
@@ -337,24 +336,37 @@ void ThumbnailProtocol::get(const QUrl &url)
     finished();
 }
 
-QString ThumbnailProtocol::pluginForMimeType(const QString& mimeType) {
-    KService::List offers = KMimeTypeTrader::self()->query( mimeType, QLatin1String("ThumbCreator"));
-    if (!offers.isEmpty()) {
-        KService::Ptr serv;
-        serv = offers.first();
-        return serv->library();
+static QVector<KPluginMetaData> availablePlugins()
+{
+    auto jsonMetaDataPlugins = KPluginMetaData::findPlugins(QStringLiteral("kf5/thumbcreator"));
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    // TODO KF6 remove this compat codepath
+    const KService::List plugins = KServiceTypeTrader::self()->query(QStringLiteral("ThumbCreator"));
+    for (const auto &plugin : plugins) {
+        if (KPluginInfo info(plugin); info.isValid()) {
+            jsonMetaDataPlugins << info.toMetaData();
+        }
     }
+    QT_WARNING_POP
+    return jsonMetaDataPlugins;
+}
 
-    //Match group mimetypes
-    ///@todo Move this into some central location together with the related matching code in previewjob.cpp. This doesn't handle inheritance and such
-    const KService::List plugins = KServiceTypeTrader::self()->query("ThumbCreator");
-    for (const KService::Ptr& plugin : plugins) {
-        const QStringList mimeTypes = plugin->serviceTypes();
+QString ThumbnailProtocol::pluginForMimeType(const QString& mimeType) {
+    const static auto plugins = availablePlugins();
+    for (const auto &plugin : plugins) {
+        if (plugin.supportsMimeType(mimeType)) {
+            return plugin.fileName();
+        }
+    }
+    for (const auto &plugin : plugins) {
+        const QStringList mimeTypes = plugin.mimeTypes() + plugin.value(QStringLiteral("ServiceTypes"), QStringList());
         for (const QString& mime : mimeTypes) {
             if(mime.endsWith('*')) {
                 const auto mimeGroup = mime.leftRef(mime.length()-1);
                 if(mimeType.startsWith(mimeGroup))
-                    return plugin->library();
+                    return plugin.fileName();
             }
         }
     }
@@ -663,21 +675,29 @@ ThumbCreatorWithMetadata* ThumbnailProtocol::getThumbCreator(const QString& plug
 
     ThumbCreatorWithMetadata *thumbCreator = nullptr;
     if (creator) {
-        const KService::List plugins = KServiceTypeTrader::self()->query(QStringLiteral("ThumbCreator"), QStringLiteral("Library == '%1'").arg(plugin));
-        if (plugins.size() == 0) {
+        KPluginMetaData data;
+        if (plugin.contains(QLatin1String("kf5") + QDir::separator() + QLatin1String("thumbcreator"))) {
+            data = KPluginMetaData(plugin);
+        } else {
+            QT_WARNING_PUSH
+            QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+            QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+            // TODO KF6 remove this compat codepath
+            const QString constraint = QStringLiteral("Library == '%1'").arg(QFileInfo(plugin).fileName());
+            const KService::List plugins = KServiceTypeTrader::self()->query(QStringLiteral("ThumbCreator"), constraint);
+            if (!plugins.isEmpty()) {
+                data = KPluginInfo(plugins.first()).toMetaData();
+            }
+            QT_WARNING_POP
+        }
+        if (!data.isValid()) {
             qCWarning(KIO_THUMBNAIL_LOG) << "Plugin not found:" << plugin;
         } else {
-            auto service = plugins.first();
-
-            QVariant cacheThumbnails = service->property("CacheThumbnail");
-            QVariant devicePixelRatioDependent = service->property("DevicePixelRatioDependent");
-            QVariant handleSequences = service->property("HandleSequences");
-
             thumbCreator = new ThumbCreatorWithMetadata{
-                creator,
-                cacheThumbnails.isValid() ? cacheThumbnails.toBool() : true,
-                devicePixelRatioDependent.isValid() ? devicePixelRatioDependent.toBool() : false,
-                handleSequences.isValid() ? handleSequences.toBool() : false
+                std::unique_ptr<ThumbCreator>(creator),
+                data.value("CacheThumbnail", true),
+                data.value("DevicePixelRatioDependent", false),
+                data.value("HandleSequences", false),
             };
         }
     } else {
@@ -826,7 +846,7 @@ bool ThumbnailProtocol::createThumbnail(ThumbCreatorWithMetadata* thumbCreator,
     if (thumbCreator->devicePixelRatioDependent) {
         KIO::ThumbDevicePixelRatioDependentCreator *dprDependentCreator =
                 static_cast<KIO::ThumbDevicePixelRatioDependentCreator *>(
-                    thumbCreator->creator);
+                    thumbCreator->creator.get());
 
         if (dprDependentCreator) {
             dprDependentCreator->setDevicePixelRatio(m_devicePixelRatio);
